# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.address_object import Address
from cybox.objects.http_session_object import HTTPSession
from cybox.objects.uri_object import URI
from cybox.test.objects import ObjectTestCase


class TestHTTPSession(ObjectTestCase, unittest.TestCase):
    object_type = "HTTPSessionObjectType"
    klass = HTTPSession

    _full_dict = {
        'http_request_response': [
            {
                'http_client_request': {
                    'http_request_line': {
                        'http_method': u("GET"),
                        'value': u("/index.html"),
                        'version': u("http/1.1"),
                    },
                    'http_request_header': {
                        'raw_header': u("...ALL THE REQUEST HEADERS..."),
                        'parsed_header': {
                            'accept': u("text/plain"),
                            'accept_charset': u("utf-8"),
                            'accept_language': u("en-US"),
                            'accept_datetime':
                                    u("Thu, 31 May 2007 20:35:00 GMT"),
                            'accept_encoding': u("gzip, deflate"),
                            'authorization': u("Basic QWxhZGRbpjpvc"),
                            'cache_control': u("no-cache"),
                            'connection': u("keep-alive"),
                            'cookie': u("$Version=1; Skin=new;"),
                            'content_length': 348,
                            'content_md5': u("Q2hlY2sgSW50ZWdyaXR5IQ=="),
                            'content_type': u("application/xml"),
                            'date': "1994-11-15T08:12:31+00:00",
                            'expect': u("200-ok"),
                            'from': {'address_value': u("bob@example.com"),
                                        'category': Address.CAT_EMAIL,
                                        'xsi:type': "AddressObjectType"},
                            'host': {
                                'domain_name': {
                                    'value': u("en.wikipedia.org"),
                                    'type': URI.TYPE_DOMAIN,
                                    'xsi:type': "URIObjectType"
                                },
                                'port': {
                                    'port_value': 80,
                                    'layer4_protocol': u("TCP"),
                                    'xsi:type': "PortObjectType"
                                }
                            },
                            'if_match':
                                    u("737060cd8c284d8af7ad3082f209582d"),
                            'if_modified_since':
                                    "1994-10-29T19:43:31+00:00",
                            'if_none_match':
                                    u("737060cd8c284d8af7ad3082f209582e"),
                            'if_range':
                                    u("737060cd8c284d8af7ad3082f209582f"),
                            'if_unmodified_since':
                                    "1996-11-22T09:33:32+00:00",
                            'max_forwards': 10,
                            'pragma': u("no-cache"),
                            'proxy_authorization': u("Basic QwxhZ=="),
                            'range': u("bytes=500-999"),
                            'referer': {
                                'value': u("http://en.wikipedia.org/wiki"),
                                'type': URI.TYPE_URL,
                                'xsi:type': "URIObjectType",
                            },
                            'te': u("trailers, deflate"),
                            'user_agent': u("Mozilla/5.0 Firefox/21.0"),
                            'via': u("1.0 fred, 1.1 example.com"),
                            'warning': u("199 Miscellaneous warning"),
                            'dnt': u("1 (Do Not Track Enabled)"),
                            'x_requested_with': u("XMLHttpRequest"),
                            'x_forwarded_for': u("client1, proxy1"),
                            'x_forwarded_proto': u("https"),
                            'x_att_deviceid': u("MakeModel/Firmware"),
                            'x_wap_profile': {
                                'value': u("http://samsung.com/SGHI777.xml"),
                                'type': URI.TYPE_URL,
                                'xsi:type': "URIObjectType",
                            },
                        }
                    },
                    'http_message_body': {
                        'length': 10,
                        'message_body': u("Hi there!!"),
                    }
                },
            },
            {
                'http_server_response': {
                    'http_status_line': {
                        'version': u("http/1.0"),
                        'status_code': 200,
                        'reason_phrase': u("OK")
                    },
                    'http_response_header': {
                        'raw_header': u("...ALL THE RESPONSE HEADERS..."),
                        'parsed_header': {
                            'access_control_allow_origin': u("*"),
                            'accept_ranges': u("bytes"),
                            'age': 12,
                            'cache_control': u("max-age=3600"),
                            'connection': u("close"),
                            'content_encoding': u("gzip"),
                            'content_language': u("da"),
                            'content_length': 348,
                            'content_location': u("/index.htm"),
                            'content_md5': u("Q2hlY2sgSW50ZWdyaXR5IQ=="),
                            'content_disposition': u("attachment; filename"),
                            'content_range': u("bytes 21010-47021"),
                            'content_type': u("text/html; charset=utf-8"),
                            'date': "1994-11-15T08:12:31+00:00",
                            'etag': u('"737060cd8c284d8af7ad3082f20"'),
                            'expires': "1994-12-01T16:00:00+00:00",
                            'last_modified': "1994-11-15T16:00:00+00:00",
                            'link': u('1</feed>; rel="alternate"'),
                            'location': {
                                'value': u("http://www.w3c.org/pub/hi.html"),
                                'type': URI.TYPE_URL,
                                'xsi:type': "URIObjectType",
                            },
                            'p3p': u("CP=\"This is not a P3P policy!\""),
                            'pragma': u("no-cache"),
                            'proxy_authenticate': u("Basic"),
                            'refresh': u("5"),
                            'retry_after': 120,
                            'server': u("Apache/2.4.1 (Unix)"),
                            'set_cookie': u("UserID=JohnDoe, Version=1"),
                            'strict_transport_security': u("max-age=160740"),
                            'trailer': u("Max-Forwards"),
                            'transfer_encoding': u("chunked"),
                            'vary': u("*"),
                            'via': u("1.0 fred, 1.1 example.com"),
                            'warning': u("199 Miscellaneous warning"),
                            'www_authenticate': u("Basic"),
                            'x_frame_options': u("deny"),
                            'x_xss_protection': u("1; mode=block"),
                            'x_content_type_options': u("nosniff"),
                            'x_powered_by': u("PHP/5.4.0"),
                            'x_ua_compatible': u("Chrome=1"),
                        }
                    },
                    'http_message_body': {
                        'length': 26,
                        'message_body': u("<html><head></head></html>"),
                    }
                },
            },
        ],
        'xsi:type': object_type,
    }

    def test_object_reference(self, ref_dict=None):
        # We have to put at least some content in here, since at least one
        # HTTPRequestResponse is required by the bindings for the round trip.
        sess_dict = {
            'http_request_response': [{
                    'http_client_request': {
                        'http_request_line': {'http_method': u("GET")}
                    }
            }]
        }
        ObjectTestCase.test_object_reference(self, sess_dict)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import json
import logging
import unittest
import collections

from mixbox.binding_utils import ExternalEncoding
from mixbox.entities import Entity
from mixbox.vendor import six

import cybox.utils

logger = logging.getLogger(__name__)


def assert_equal_ignore(item1, item2, ignore_keys=None):
    """Recursively compare two dictionaries, ignoring differences in some keys.
    """
    if not ignore_keys:
        ignore_keys = []

    if isinstance(item1, dict) and isinstance(item2, dict):
        item1keys = set(item1.keys())
        item2keys = set(item2.keys())
        ignore = set(ignore_keys)
        compare_keys = (item1keys | item2keys) - ignore
        for k in compare_keys:
            assert k in item1, "Item 1 is missing %s" % k
            assert k in item2, "Item 2 is missing %s" % k
            assert_equal_ignore(item1.get(k), item2.get(k), ignore_keys)
    elif isinstance(item1, list) and isinstance(item2, list):
        assert len(item1) == len(item2), "Lists are of different lengths"
        for (x, y) in zip(item1, item2):
            assert_equal_ignore(x, y, ignore_keys)
    else:
        assert item1 == item2, "%s != %s" % (item1, item2)


def assert_entity_equals(entity, other, name=None, stack=None):
    """Assert all of the TypedFields in two Entities are equal."""
    # Shorten the lines.
    is_entity      = lambda x: isinstance(x, Entity)
    is_mutableseq  = lambda x: isinstance(x, collections.MutableSequence)

    if stack is None:
        stack = []

    if name is not None:
        stack.append(name)

    if is_entity(entity) and is_entity(other):
        for name, var in entity.typed_fields_with_attrnames():
            assert_entity_equals(
                var.__get__(entity),
                var.__get__(other),
                name=name,
                stack=stack
            )
    elif is_mutableseq(entity) and is_mutableseq(other):
        # "multiple" TypedFields store their contents in mutable sequences.
        assert len(entity) == len(other)
        for x, y in zip(entity, other):
            assert_entity_equals(x, y, None, stack=stack)
    else:
        assert entity == other, "(%s) %r != %r stack=%s" % (name, entity, other, stack)

    if name is not None and stack:
        stack.pop()

def round_trip(o, output=False, list_=False):
    """ Performs all eight conversions to verify import/export functionality.

    1. cybox.Entity -> dict/list
    2. dict/list -> JSON string
    3. JSON string -> dict/list
    4. dict/list -> cybox.Entity
    5. cybox.Entity -> Bindings Object
    6. Bindings Object -> XML String
    7. XML String -> Bindings Object
    8. Bindings object -> cybox.Entity

    It returns the final object, so tests which call this function can check to
    ensure it was not modified during any of the transforms.
    """

    klass = o.__class__
    if output:
        logger.debug("Class: {0}".format(klass))
        logger.debug("-" * 40)

    # 1. cybox.Entity -> dict/list
    if list_:
        d = o.to_list()
    else:
        d = o.to_dict()

    # 2. dict/list -> JSON string
    json_string = json.dumps(d)

    if output:
        logger.debug(json_string)
        logger.debug("-" * 40)

    # Before parsing the JSON, make sure the cache is clear
    cybox.utils.cache_clear()

    # 3. JSON string -> dict/list
    d2 = json.loads(json_string)

    # 4. dict/list -> cybox.Entity
    if list_:
        o2 = klass.from_list(d2)
    else:
        o2 = klass.from_dict(d2)

    # 5. cybox.Entity -> Bindings Object
    xobj = o2.to_obj()

    # 6. Bindings Object -> XML String
    xml_string = o2.to_xml(encoding=ExternalEncoding)

    # Explicitly check to see if it's a Unicode string before trying to decode
    # it.
    if not isinstance(xml_string, six.text_type):
        xml_string = xml_string.decode(ExternalEncoding)

    if output:
        logger.debug(xml_string)
        logger.debug("-" * 40)

    # Before parsing the XML, make sure the cache is clear
    cybox.utils.cache_clear()

    #7. XML String -> Bindings Object
    xobj2 = klass._binding.parseString(xml_string)

    # 8. Bindings object -> cybox.Entity
    o3 = klass.from_obj(xobj2)

    return o3


def round_trip_dict(cls, dict_):
    obj = cls.object_from_dict(dict_)
    dict2 = cls.dict_from_object(obj)

    return dict2


def round_trip_list(cls, list_):
    obj = cls.object_from_list(list_)
    list2 = cls.list_from_object(obj)

    return list2


class EntityTestCase(object):
    """A mixin class for testing CybOX Entities"""

    def setUp(self):
        self.assertNotEqual(self.klass, None)
        self.assertNotEqual(self._full_dict, None)

    def test_round_trip_dict(self):
        # Round_trip_dict doesn't start or end with Python objects (obviously),
        # so this is a less than ideal test.
        dict2 = round_trip_dict(self.klass, self._full_dict)
        self.maxDiff = None
        self.assertEqual(self._full_dict, dict2)

    def test_round_trip(self):
        # This is a better test, even though we start from a dictionary, and
        # can only compare the dict representations right now.
        ent = self.klass.from_dict(self._full_dict)
        ent2 = round_trip(ent, output=True)
        self.maxDiff = None
        # For now, the only way to compare two entity representations is to
        # compare the dictionary output to the original dictionary.
        self.assertEqual(self._full_dict, ent2.to_dict())

    def test_round_trip_entity(self):
        # This is a better test, even though we start from a dictionary, and
        # can only compare the dict representations right now.
        ent = self.klass.from_dict(self._full_dict)
        ent2 = round_trip(ent, output=True)
        assert_entity_equals(ent, ent2)

    # TODO: Assert to_xml() on two round-tripped entities are identical.


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_handle_object as win_handle_binding
from cybox.common import (ObjectProperties, String, UnsignedLong,
        UnsignedInteger)


class WinHandle(ObjectProperties):
    _binding = win_handle_binding
    _binding_class = win_handle_binding.WindowsHandleObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-handle-2'
    _XSI_NS = "WinHandleObj"
    _XSI_TYPE = "WindowsHandleObjectType"

    id_ = fields.TypedField("ID", UnsignedInteger)
    name = fields.TypedField("Name", String)
    type_ = fields.TypedField("Type", String)
    object_address = fields.TypedField("Object_Address", UnsignedLong)
    access_mask = fields.TypedField("Access_Mask", UnsignedLong)
    pointer_count = fields.TypedField("Pointer_Count", UnsignedLong)


class WinHandleList(entities.EntityList):
    _binding = win_handle_binding
    _binding_class = win_handle_binding.WindowsHandleListType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-handle-2'
    handle = fields.TypedField("Handle", WinHandle, multiple=True)


# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common


class SSDTEntryListType(GeneratedsSuper):
    """The SSDTEntryListType type specifies a listing of the entries in the
    System Service Descriptor Table (SSDT)."""

    subclass = None
    superclass = None
    def __init__(self, SSDT_Entry=None):
        if SSDT_Entry is None:
            self.SSDT_Entry = []
        else:
            self.SSDT_Entry = SSDT_Entry
    def factory(*args_, **kwargs_):
        if SSDTEntryListType.subclass:
            return SSDTEntryListType.subclass(*args_, **kwargs_)
        else:
            return SSDTEntryListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_SSDT_Entry(self): return self.SSDT_Entry
    def set_SSDT_Entry(self, SSDT_Entry): self.SSDT_Entry = SSDT_Entry
    def add_SSDT_Entry(self, value): self.SSDT_Entry.append(value)
    def insert_SSDT_Entry(self, index, value): self.SSDT_Entry[index] = value
    def hasContent_(self):
        if (
            self.SSDT_Entry
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelObj:', name_='SSDTEntryListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='SSDTEntryListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelObj:', name_='SSDTEntryListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinKernelObj:', name_='SSDTEntryListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for SSDT_Entry_ in self.SSDT_Entry:
            SSDT_Entry_.export(lwrite, level, 'WinKernelObj:', name_='SSDT_Entry', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'SSDT_Entry':
            obj_ = SSDTEntryType.factory()
            obj_.build(child_)
            self.SSDT_Entry.append(obj_)
# end class SSDTEntryListType

class SSDTEntryType(GeneratedsSuper):
    """The SSDTEntryType type specifies a single entry in the System
    Service Descriptor Table (SSDT).The hooked attribute specifies
    whether the SSDT entry is hooked."""

    subclass = None
    superclass = None
    def __init__(self, hooked=None, Service_Table_Base=None, Service_Counter_Table_Base=None, Number_Of_Services=None, Argument_Table_Base=None):
        self.hooked = _cast(bool, hooked)
        self.Service_Table_Base = Service_Table_Base
        self.Service_Counter_Table_Base = Service_Counter_Table_Base
        self.Number_Of_Services = Number_Of_Services
        self.Argument_Table_Base = Argument_Table_Base
    def factory(*args_, **kwargs_):
        if SSDTEntryType.subclass:
            return SSDTEntryType.subclass(*args_, **kwargs_)
        else:
            return SSDTEntryType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Service_Table_Base(self): return self.Service_Table_Base
    def set_Service_Table_Base(self, Service_Table_Base): self.Service_Table_Base = Service_Table_Base
    def validate_HexBinaryObjectPropertyType(self, value):
        # Validate type cybox_common.HexBinaryObjectPropertyType, a restriction on None.
        pass
    def get_Service_Counter_Table_Base(self): return self.Service_Counter_Table_Base
    def set_Service_Counter_Table_Base(self, Service_Counter_Table_Base): self.Service_Counter_Table_Base = Service_Counter_Table_Base
    def get_Number_Of_Services(self): return self.Number_Of_Services
    def set_Number_Of_Services(self, Number_Of_Services): self.Number_Of_Services = Number_Of_Services
    def validate_NonNegativeIntegerObjectPropertyType(self, value):
        # Validate type cybox_common.NonNegativeIntegerObjectPropertyType, a restriction on None.
        pass
    def get_Argument_Table_Base(self): return self.Argument_Table_Base
    def set_Argument_Table_Base(self, Argument_Table_Base): self.Argument_Table_Base = Argument_Table_Base
    def get_hooked(self): return self.hooked
    def set_hooked(self, hooked): self.hooked = hooked
    def hasContent_(self):
        if (
            self.Service_Table_Base is not None or
            self.Service_Counter_Table_Base is not None or
            self.Number_Of_Services is not None or
            self.Argument_Table_Base is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelObj:', name_='SSDTEntryType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='SSDTEntryType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelObj:', name_='SSDTEntryType'):
        if self.hooked is not None:

            lwrite(' hooked="%s"' % self.gds_format_boolean(self.hooked, input_name='hooked'))
    def exportChildren(self, lwrite, level, namespace_='WinKernelObj:', name_='SSDTEntryType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Service_Table_Base is not None:
            self.Service_Table_Base.export(lwrite, level, 'WinKernelObj:', name_='Service_Table_Base', pretty_print=pretty_print)
        if self.Service_Counter_Table_Base is not None:
            self.Service_Counter_Table_Base.export(lwrite, level, 'WinKernelObj:', name_='Service_Counter_Table_Base', pretty_print=pretty_print)
        if self.Number_Of_Services is not None:
            self.Number_Of_Services.export(lwrite, level, 'WinKernelObj:', name_='Number_Of_Services', pretty_print=pretty_print)
        if self.Argument_Table_Base is not None:
            self.Argument_Table_Base.export(lwrite, level, 'WinKernelObj:', name_='Argument_Table_Base', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('hooked', node)
        if value is not None:

            if value in ('true', '1'):
                self.hooked = True
            elif value in ('false', '0'):
                self.hooked = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Service_Table_Base':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Service_Table_Base(obj_)
        elif nodeName_ == 'Service_Counter_Table_Base':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Service_Counter_Table_Base(obj_)
        elif nodeName_ == 'Number_Of_Services':
            obj_ = cybox_common.NonNegativeIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Number_Of_Services(obj_)
        elif nodeName_ == 'Argument_Table_Base':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Argument_Table_Base(obj_)
# end class SSDTEntryType

class IDTEntryListType(GeneratedsSuper):
    """The IDTEntryListType type specifies a listing of the entries in the
    Interrupt Descriptor Table (IDT). The IDT is specific to the
    I386 architecture, indicating where the Prtoetcted mode
    Interrupt Service Routines (ISR) are located. See
    http://wiki.osdev.org/Interrupt_Descriptor_Table"""

    subclass = None
    superclass = None
    def __init__(self, IDT_Entry=None):
        if IDT_Entry is None:
            self.IDT_Entry = []
        else:
            self.IDT_Entry = IDT_Entry
    def factory(*args_, **kwargs_):
        if IDTEntryListType.subclass:
            return IDTEntryListType.subclass(*args_, **kwargs_)
        else:
            return IDTEntryListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_IDT_Entry(self): return self.IDT_Entry
    def set_IDT_Entry(self, IDT_Entry): self.IDT_Entry = IDT_Entry
    def add_IDT_Entry(self, value): self.IDT_Entry.append(value)
    def insert_IDT_Entry(self, index, value): self.IDT_Entry[index] = value
    def hasContent_(self):
        if (
            self.IDT_Entry
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelObj:', name_='IDTEntryListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='IDTEntryListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelObj:', name_='IDTEntryListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinKernelObj:', name_='IDTEntryListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for IDT_Entry_ in self.IDT_Entry:
            IDT_Entry_.export(lwrite, level, 'WinKernelObj:', name_='IDT_Entry', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'IDT_Entry':
            obj_ = IDTEntryType.factory()
            obj_.build(child_)
            self.IDT_Entry.append(obj_)
# end class IDTEntryListType

class IDTEntryType(GeneratedsSuper):
    """The IDTEntryType type specifies a single entry in the Interrupt
    Descriptor Table (IDT). Entries can be interrupt gates, task
    gates, and trap gates."""

    subclass = None
    superclass = None
    def __init__(self, Type_Attr=None, Offset_High=None, Offset_Low=None, Offset_Middle=None, Selector=None):
        self.Type_Attr = Type_Attr
        self.Offset_High = Offset_High
        self.Offset_Low = Offset_Low
        self.Offset_Middle = Offset_Middle
        self.Selector = Selector
    def factory(*args_, **kwargs_):
        if IDTEntryType.subclass:
            return IDTEntryType.subclass(*args_, **kwargs_)
        else:
            return IDTEntryType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Type_Attr(self): return self.Type_Attr
    def set_Type_Attr(self, Type_Attr): self.Type_Attr = Type_Attr
    def validate_HexBinaryObjectPropertyType(self, value):
        # Validate type cybox_common.HexBinaryObjectPropertyType, a restriction on None.
        pass
    def get_Offset_High(self): return self.Offset_High
    def set_Offset_High(self, Offset_High): self.Offset_High = Offset_High
    def get_Offset_Low(self): return self.Offset_Low
    def set_Offset_Low(self, Offset_Low): self.Offset_Low = Offset_Low
    def get_Offset_Middle(self): return self.Offset_Middle
    def set_Offset_Middle(self, Offset_Middle): self.Offset_Middle = Offset_Middle
    def get_Selector(self): return self.Selector
    def set_Selector(self, Selector): self.Selector = Selector
    def hasContent_(self):
        if (
            self.Type_Attr is not None or
            self.Offset_High is not None or
            self.Offset_Low is not None or
            self.Offset_Middle is not None or
            self.Selector is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelObj:', name_='IDTEntryType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='IDTEntryType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelObj:', name_='IDTEntryType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinKernelObj:', name_='IDTEntryType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Type_Attr is not None:
            self.Type_Attr.export(lwrite, level, 'WinKernelObj:', name_='Type_Attr', pretty_print=pretty_print)
        if self.Offset_High is not None:
            self.Offset_High.export(lwrite, level, 'WinKernelObj:', name_='Offset_High', pretty_print=pretty_print)
        if self.Offset_Low is not None:
            self.Offset_Low.export(lwrite, level, 'WinKernelObj:', name_='Offset_Low', pretty_print=pretty_print)
        if self.Offset_Middle is not None:
            self.Offset_Middle.export(lwrite, level, 'WinKernelObj:', name_='Offset_Middle', pretty_print=pretty_print)
        if self.Selector is not None:
            self.Selector.export(lwrite, level, 'WinKernelObj:', name_='Selector', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Type_Attr':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Type_Attr(obj_)
        elif nodeName_ == 'Offset_High':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Offset_High(obj_)
        elif nodeName_ == 'Offset_Low':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Offset_Low(obj_)
        elif nodeName_ == 'Offset_Middle':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Offset_Middle(obj_)
        elif nodeName_ == 'Selector':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Selector(obj_)
# end class IDTEntryType

class WindowsKernelObjectType(cybox_common.ObjectPropertiesType):
    """The WindowsKernelObjectType type is intended to characterize Windows
    Kernel structures."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, IDT=None, SSDT=None):
        super(WindowsKernelObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.IDT = IDT
        self.SSDT = SSDT
    def factory(*args_, **kwargs_):
        if WindowsKernelObjectType.subclass:
            return WindowsKernelObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsKernelObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_IDT(self): return self.IDT
    def set_IDT(self, IDT): self.IDT = IDT
    def get_SSDT(self): return self.SSDT
    def set_SSDT(self, SSDT): self.SSDT = SSDT
    def hasContent_(self):
        if (
            self.IDT is not None or
            self.SSDT is not None or
            super(WindowsKernelObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelObj:', name_='WindowsKernelObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsKernelObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelObj:', name_='WindowsKernelObjectType'):
        super(WindowsKernelObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsKernelObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinKernelObj:', name_='WindowsKernelObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsKernelObjectType, self).exportChildren(lwrite, level, 'WinKernelObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.IDT is not None:
            self.IDT.export(lwrite, level, 'WinKernelObj:', name_='IDT', pretty_print=pretty_print)
        if self.SSDT is not None:
            self.SSDT.export(lwrite, level, 'WinKernelObj:', name_='SSDT', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsKernelObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'IDT':
            obj_ = IDTEntryListType.factory()
            obj_.build(child_)
            self.set_IDT(obj_)
        elif nodeName_ == 'SSDT':
            obj_ = SSDTEntryListType.factory()
            obj_.build(child_)
            self.set_SSDT(obj_)
        super(WindowsKernelObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsKernelObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Segments': cybox_common.HashSegmentsType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Selector': cybox_common.HexBinaryObjectPropertyType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Argument_Table_Base': cybox_common.HexBinaryObjectPropertyType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Number_Of_Services': cybox_common.NonNegativeIntegerObjectPropertyType,
    'Contributors': cybox_common.PersonnelType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Type_Attr': cybox_common.HexBinaryObjectPropertyType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Data_Size': cybox_common.DataSizeType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Offset_High': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Offset_Low': cybox_common.HexBinaryObjectPropertyType,
    'Hashes': cybox_common.HashListType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Offset_Middle': cybox_common.HexBinaryObjectPropertyType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Service_Counter_Table_Base': cybox_common.HexBinaryObjectPropertyType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Service_Table_Base': cybox_common.HexBinaryObjectPropertyType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Date': cybox_common.DateRangeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Kernel'
        rootClass = WindowsKernelObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Kernel'
        rootClass = WindowsKernelObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Kernel'
        rootClass = WindowsKernelObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Windows_Kernel",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsKernelObjectType",
    "SSDTEntryListType",
    "SSDTEntryType",
    "IDTEntryListType",
    "IDTEntryType"
    ]



#!/usr/bin/env python

# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

'''A collection of functions, etc. that are useful during development.'''


import os
import sys

from mixbox import entities

# This is not in the Python Standard Library before Python 2.7
import importlib


def main():
    import_mods(".")

    subs = list(subclasses(entities.Entity))

#    print("\n".join([str(x) for x in subs]))
#    print(len(subs))

    no_namespace = [x for x in subs if not filter_has_namespace(x)]
    for x in no_namespace:
        print(x)
    print(len(no_namespace))


def filter_has_namespace(cls):
    if hasattr(cls, "_namespace"):
        return True
    return False


#Inpsired by http://code.activestate.com/recipes/576949-find-all-subclasses-of-a-given-class/
def subclasses(cls, _seen=None):
    """Generator to return all subclasses of a given class"""

    if _seen is None:
        _seen = set()

    for sub in cls.__subclasses__():
        if sub not in _seen:
            _seen.add(sub)
            yield sub
            for sub in subclasses(sub, _seen):
                yield sub


def import_mods(directory):
    """Recursively import all python modules in a given directory"""
    for root, dirs, files in os.walk(directory):
        if '.git' in dirs:
            dirs.remove('.git')

        if 'examples' in dirs:
            dirs.remove('examples')

        for fn in files:
            filename = os.path.join(root, fn)

            if filename.endswith(".py"):
                #print filename

                modname = filename[2:].replace("/", ".")[:-3]
                #print modname

                if modname in ('setup',):
                    continue

                m = importlib.import_module(modname)
                #print m

if __name__ == "__main__":
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.win_kernel_hook_object import WinKernelHook
from cybox.test.objects import ObjectTestCase


class TestWinKernelHook(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsKernelHookObjectType"
    klass = WinKernelHook

    _full_dict = {
        'digital_signature_hooking': {
                'signature_exists': True,
                'signature_verified': False,
                'signature_description': "Sig of hooking code",
            },
        'digital_signature_hooked': {
                'signature_verified': True,
                'signature_description': "Sig of hooked code",
            },
        'hooking_address': 0xc001d1ce,
        'hook_description': "A powerful left hook",
        'hooked_function': "DoSomeStuff",
        'hooked_module': "Some Module",
        'hooking_module': "Another Module",
        'type': "Instruction_Hooking",
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.hostname_object as hostname_binding
from cybox.common import ObjectProperties, String


class Hostname(ObjectProperties):
    _binding = hostname_binding
    _binding_class = hostname_binding.HostnameObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/hostname-1"
    _XSI_NS = "HostnameObj"
    _XSI_TYPE = "HostnameObjectType"

    is_domain_name = fields.TypedField("is_domain_name")
    hostname_value = fields.TypedField("Hostname_Value", String)
    naming_system = fields.TypedField("Naming_System", String, multiple=True)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_file_object as win_file_binding
from cybox.objects.file_object import File, FileAttribute, FilePermissions
from cybox.common import DateTime, Hash, String, UnsignedLong


class Stream(entities.Entity):
    _binding = win_file_binding
    _binding_class = win_file_binding.StreamObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-file-2'

    name = fields.TypedField("Name", String)
    size_in_bytes = fields.TypedField("Size_In_Bytes", UnsignedLong)
    hashes = fields.TypedField("Hash", Hash, key_name="hashes", multiple=True)


class StreamList(entities.EntityList):
    _binding = win_file_binding
    _binding_class = win_file_binding.StreamListType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-file-2'
    stream = fields.TypedField("Stream", Stream, multiple=True)


class WindowsFileAttribute(String):
    _binding = win_file_binding
    _binding_class = win_file_binding.WindowsFileAttributeType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-file-2'


class WindowsFileAttributes(FileAttribute, entities.EntityList):
    _binding = win_file_binding
    _binding_class = win_file_binding.WindowsFileAttributesType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-file-2'
    attribute = fields.TypedField("Attribute", WindowsFileAttribute, multiple=True)


class WindowsFilePermissions(FilePermissions):
    _binding = win_file_binding
    _binding_class = win_file_binding.WindowsFilePermissionsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-file-2'

    full_control = fields.TypedField("Full_Control")
    modify = fields.TypedField("Modify")
    read = fields.TypedField("Read")
    read_and_execute = fields.TypedField("Read_And_Execute")
    write = fields.TypedField("Write")


class WinFile(File):
    _binding = win_file_binding
    _binding_class = win_file_binding.WindowsFileObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-file-2'
    _XSI_NS = "WinFileObj"
    _XSI_TYPE = "WindowsFileObjectType"

    filename_accessed_time = fields.TypedField("Filename_Accessed_Time",
                                              DateTime)
    filename_created_time = fields.TypedField("Filename_Created_Time", DateTime)
    filename_modified_time = fields.TypedField("Filename_Modified_Time",
                                              DateTime)
    drive = fields.TypedField("Drive", String)
    security_id = fields.TypedField("Security_ID", String)
    security_type = fields.TypedField("Security_Type", String)
    stream_list = fields.TypedField("Stream_List", StreamList)

    #Override abstract types
    file_attributes_list = fields.TypedField('File_Attributes_List',
                                            WindowsFileAttributes)
    permissions = fields.TypedField('Permissions', WindowsFilePermissions)

    @property
    def privilege_list(self):
        return self.permissions

    @privilege_list.setter
    def privilege_list(self, value):
        self.permissions = value



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_event_log_object as win_event_log_binding
from cybox.common import ObjectProperties, String, Base64Binary, DateTime, Long 


class UnformattedMessageList(entities.EntityList):
    _binding = win_event_log_binding
    _binding_class = win_event_log_binding.UnformattedMessageListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-event-log-2"
    unformatted_message = fields.TypedField("Unformatted_Message", String, multiple=True)


class WinEventLog(ObjectProperties):
    _binding = win_event_log_binding
    _binding_class = win_event_log_binding.WindowsEventLogObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-event-log-2"
    _XSI_NS = "WinEventLogObj"
    _XSI_TYPE = "WindowsEventLogObjectType"

    eid = fields.TypedField("EID", Long)
    type_ = fields.TypedField("Type", String)
    log = fields.TypedField("Log", String)
    message = fields.TypedField("Message", String)
    category_num = fields.TypedField("Category_Num", Long)
    category = fields.TypedField("Category", String)
    generation_time = fields.TypedField("Generation_Time", DateTime)
    source = fields.TypedField("Source", String)
    machine = fields.TypedField("Machine", String)
    user = fields.TypedField("User", String)
    blob = fields.TypedField("Blob", Base64Binary)
    correlation_activity_id = fields.TypedField("Correlation_Activity_ID", String)
    correlation_related_activity_id = fields.TypedField("Correlation_Related_Activity_ID", String)
    execution_process_id = fields.TypedField("Execution_Process_ID", String)
    execution_thread_id = fields.TypedField("Execution_Thread_ID", String)
    index = fields.TypedField("Index", Long)
    reserved = fields.TypedField("Reserved", Long)
    unformatted_message_list = fields.TypedField("Unformatted_Message_List", UnformattedMessageList)
    write_time = fields.TypedField("Write_Time", DateTime)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.library_object as library_binding
from cybox.common.extracted_features import ExtractedFeatures
from cybox.common import ObjectProperties, String, UnsignedLong, HexBinary


class Library(ObjectProperties):
    _binding = library_binding
    _binding_class = library_binding.LibraryObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/library-2"
    _XSI_NS = "LibraryObj"
    _XSI_TYPE = "LibraryObjectType"

    name = fields.TypedField("Name", String)
    path = fields.TypedField("Path", String)
    size = fields.TypedField("Size", UnsignedLong)
    type_ = fields.TypedField("Type", String)
    version = fields.TypedField("Version", String)
    base_address = fields.TypedField("Base_Address", HexBinary)
    extracted_features = fields.TypedField("Extracted_Features", ExtractedFeatures)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
from mixbox import entities
from mixbox import fields
from mixbox.vendor import six

import cybox.bindings.cybox_common as common_binding
from cybox.common.attribute_groups import PatternFieldGroup
from cybox.common.datetimewithprecision import (validate_date_precision,
    validate_time_precision, validate_datetime_precision)
from cybox.utils import normalize_to_xml, denormalize_from_xml

DATE_PRECISION_VALUES = ("year", "month", "day")
TIME_PRECISION_VALUES = ("hour", "minute", "second")
DATETIME_PRECISION_VALUES = DATE_PRECISION_VALUES + TIME_PRECISION_VALUES


def validate_string_type(instance, value):
    if value is None:
        return
    elif isinstance(value, six.string_types):
        return
    elif isinstance(value, list):
        for x in value:
            validate_string_type(instance, x)
    else:
        raise ValueError("Value must be a string. Received %r" % value)


class ListFieldMixin(object):
    """Mixin that allows a TypedField to be set to a list of values or a single
    value. If a list of values are passed in, each item in the list will be
    passed through the _clean() method.
    """
    def check_type(self, value):
        func_check = super(ListFieldMixin, self).check_type
        if isinstance(value, list):
            return all(func_check(x) for x in value)
        return func_check(value)

    def _clean(self, value):
        func_clean = super(ListFieldMixin, self)._clean
        if isinstance(value, list):
            return [func_clean(x) for x in value]
        return func_clean(value)


# Field types that can accept lists or scalar values.
class ListTypedField(ListFieldMixin, fields.TypedField): pass
class ListDateField(ListFieldMixin, fields.DateField): pass
class ListDateTimeField(ListFieldMixin, fields.DateTimeField): pass
class ListFloatField(ListFieldMixin, fields.FloatField): pass
class ListIntegerField(ListFieldMixin, fields.IntegerField): pass
class ListLongField(ListFieldMixin, fields.LongField): pass


@six.python_2_unicode_compatible
class BaseProperty(PatternFieldGroup, entities.Entity):
    # Most Properties are defined in the "common" binding, so we'll just set
    # that here. Some BaseProperty subclasses might have to override this.
    _binding = common_binding
    _binding_class = _binding.BaseObjectPropertyType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    # If `True`, force the "datatype" attribute to be output. This is
    # necessary in some cases
    _force_datatype = False
    default_datatype = 'string'

    # BaseObjectProperty Group
    id_ = fields.IdField("id")
    idref = fields.IdrefField("idref")
    value = ListTypedField("valueOf_", key_name="value")
    datatype = fields.TypedField("datatype")
    appears_random = fields.TypedField("appears_random")
    is_obfuscated = fields.TypedField("is_obfuscated")
    obfuscation_algorithm_ref = fields.TypedField("obfuscation_algorithm_ref")
    is_defanged = fields.TypedField("is_defanged")
    defanging_algorithm_ref = fields.TypedField("defanging_algorithm_ref")
    refanging_transform_type = fields.TypedField("refanging_transform_type")
    refanging_transform = fields.TypedField("refanging_transform")
    observed_encoding = fields.TypedField("observed_encoding")

    def __init__(self, value=None):
        super(BaseProperty, self).__init__()
        self._force_datatype = False

        self.value = value
        self.datatype = self.default_datatype

    def __str__(self):
        return six.text_type(self.value)

    def __int__(self):
        return int(self.value)

    @property
    def serialized_value(self):
        return self.value

    @property
    def values(self):
        """Allow uniform access to `value` as a list.

        This allows code like the following to always work where `obj` is a
        CybOX entity and `prop` is any BaseProperty subclass:

        ```
        for x in obj.prop.values():
            do_something(x)
        ```

        If `value` is None, this returns an empty list ([])
        If `value` is a single non-list value, it returns a single-item list.
        If `value` is a list, `values` is identical to `value`.

        NOTE: This property cannot be set. Use the `value` setter for this.
        """
        if self.value is None:
            return []
        elif isinstance(self.value, list):
            return self.value
        else:
            return [self.value]

    def __eq__(self, other):
        # None-type checking
        if not other:
            return False

        # It is possible to compare a Property to a single value if
        # the Property defines only the "value" property.
        if not isinstance(other, BaseProperty) and self.is_plain():
            return self.value == other

        return (
            self.value == other.value and
            self.id_ == other.id_ and
            self.idref == other.idref and
            self.datatype == other.datatype and
            self.appears_random == other.appears_random and
            self.is_obfuscated == other.is_obfuscated and
            self.obfuscation_algorithm_ref == other.obfuscation_algorithm_ref and
            self.is_defanged == other.is_defanged and
            self.defanging_algorithm_ref == other.defanging_algorithm_ref and
            self.refanging_transform_type == other.refanging_transform_type and
            self.refanging_transform == other.refanging_transform and
            self.observed_encoding == other.observed_encoding and

            PatternFieldGroup._conditions_equal(self, other) and

            self.bit_mask == other.bit_mask and
            self.pattern_type == other.pattern_type and
            self.regex_syntax == other.regex_syntax and
            self.is_case_sensitive == other.is_case_sensitive and
            self.has_changed == other.has_changed and
            self.trend == other.trend
        )

    def __ne__(self, other):
        return not self == other

    def is_plain(self):
        """Whether the Property can be represented as a single value.

        The `datatype` can be inferred by the particular BaseProperty subclass,
        so if `datatype` and `value` are the only non-None properties, the
        BaseProperty can be represented by a single value rather than a
        dictionary. This makes the JSON representation simpler without losing
        any data fidelity.
        """
        return (
            # ignore value
            self.id_ is None and
            self.idref is None and
            # ignore datatype
            self.appears_random is None and
            self.is_obfuscated is None and
            self.obfuscation_algorithm_ref is None and
            self.is_defanged is None and
            self.defanging_algorithm_ref is None and
            self.refanging_transform_type is None and
            self.refanging_transform is None and
            self.observed_encoding is None and
            super(BaseProperty, self).is_plain()
        )

    def __nonzero__(self):
        return (not self.is_plain()) or (self.value is not None)

    __bool__ = __nonzero__

    def _datatype_serialized_value(self):
        if self._force_datatype:
            return self.datatype
        elif self.datatype != self.default_datatype:
            return self.datatype
        else:
            return None

    def to_obj(self, ns_info=None):
        attr_obj = super(BaseProperty, self).to_obj(ns_info=ns_info)
        attr_obj.datatype = self._datatype_serialized_value()
        attr_obj.valueOf_ = normalize_to_xml(self.serialized_value,
                                             self.delimiter)
        return attr_obj

    def to_dict(self):
        if self.is_plain():
            return self.serialized_value

        attr_dict = super(BaseProperty, self).to_dict()
        attr_dict.pop("datatype", None)

        if self._datatype_serialized_value():
            attr_dict['datatype'] = self._datatype_serialized_value()
        if self.value is not None:
            attr_dict['value'] = self.serialized_value

        return attr_dict

    @classmethod
    def from_obj(cls, cls_obj):
        # Use the subclass this was called on to initialize the object

        if not cls_obj:
            return None

        # split delimited values now, before converting from bindings object to API object
        cls_obj.valueOf_ = denormalize_from_xml(cls_obj.valueOf_, cls_obj.delimiter)
        attr = super(BaseProperty, cls).from_obj(cls_obj)
        attr.datatype = cls_obj.datatype or cls.default_datatype
        return attr

    @classmethod
    def from_dict(cls, cls_dict):
        attr = super(BaseProperty, cls).from_dict(cls_dict)

        if isinstance(cls_dict, dict):
            attr.datatype = cls_dict.get('datatype', cls.default_datatype)

        return attr

class String(BaseProperty):
    _binding_class = common_binding.StringObjectPropertyType
    default_datatype = "string"
    value = ListTypedField("valueOf_", key_name="value", preset_hook=validate_string_type)


class _IntegerBase(BaseProperty):
    """Define a common _parse_value function for all Integer types"""
    value = ListIntegerField("valueOf_", key_name="value")


class Integer(_IntegerBase):
    _binding_class = common_binding.IntegerObjectPropertyType
    default_datatype = "int"


class PositiveInteger(_IntegerBase):
    _binding_class = common_binding.PositiveIntegerObjectPropertyType
    default_datatype = "positiveInteger"


class UnsignedInteger(_IntegerBase):
    _binding_class = common_binding.UnsignedIntegerObjectPropertyType
    default_datatype = "unsignedInt"


class NonNegativeInteger(_IntegerBase):
    _binding_class = common_binding.NonNegativeIntegerObjectPropertyType
    default_datatype = "nonNegativeInteger"


class AnyURI(BaseProperty):
    _binding_class = common_binding.AnyURIObjectPropertyType
    default_datatype = "anyURI"


class HexBinary(BaseProperty):
    _binding_class = common_binding.HexBinaryObjectPropertyType
    default_datatype = "hexBinary"


class Base64Binary(BaseProperty):
    _binding_class = common_binding.Base64BinaryObjectPropertyType
    default_datatype = "base64Binary"


class Duration(BaseProperty):
    _binding_class = common_binding.DurationObjectPropertyType
    default_datatype = "duration"


class Time(BaseProperty):
    _binding_class = common_binding.TimeObjectPropertyType
    default_datatype = "time"

    precision = fields.TypedField("precision", preset_hook=validate_time_precision)

    def __init__(self, value=None, precision='second'):
        super(Time, self).__init__(value=value)
        self.precision = precision


@six.python_2_unicode_compatible
class Date(BaseProperty):
    _binding_class = common_binding.DateObjectPropertyType
    default_datatype = "date"

    value = ListDateField("valueOf_", key_name="value")
    precision = fields.TypedField("precision", preset_hook=validate_date_precision)

    def __init__(self, value=None, precision='day'):
        super(Date, self).__init__(value=value)
        self.precision = precision

    def __str__(self):
        if self.value:
            return self.value.isoformat()
        return super(BaseProperty, self).__str__()

    @property
    def serialized_value(self):
        if isinstance(self.value, list):
            return [x.isoformat() for x in self.value]
        elif self.value is None:
            return None
        else:
            return self.value.isoformat()


@six.python_2_unicode_compatible
class DateTime(BaseProperty):
    _binding_class = common_binding.DateTimeObjectPropertyType
    default_datatype = "dateTime"

    value = ListDateTimeField("valueOf_", key_name="value")
    precision = fields.TypedField("precision", preset_hook=validate_datetime_precision)

    def __init__(self, value=None, precision='second'):
        super(DateTime, self).__init__(value=value)
        self.precision = precision

    def __str__(self):
        if self.value:
            return self.value.isoformat()
        return super(BaseProperty, self).__str__()

    @property
    def serialized_value(self):
        if isinstance(self.value, list):
            return [x.isoformat() for x in self.value]
        elif self.value is None:
            return None
        else:
            return self.value.isoformat()


class _FloatBase(BaseProperty):
    """Define a common _parse_value function for Float and Double types."""
    value = ListFloatField("valueOf_", key_name="value")


class Double(_FloatBase):
    _binding_class = common_binding.DoubleObjectPropertyType
    default_datatype = "double"


class Float(_FloatBase):
    _binding_class = common_binding.FloatObjectPropertyType
    default_datatype = "float"


class _LongBase(BaseProperty):
    """Define a common _parse_value function for all Long types."""
    value = ListLongField("valueOf_", key_name="value")


class Long(_LongBase):
    _binding_class = common_binding.LongObjectPropertyType
    default_datatype = "long"


class UnsignedLong(_LongBase):
    _binding_class = common_binding.UnsignedLongObjectPropertyType
    default_datatype = "unsignedLong"


class Name(BaseProperty):
    _binding_class = common_binding.NameObjectPropertyType
    default_datatype = "name"


# Mapping of binding classes to the corresponding BaseProperty subclass
BINDING_CLASS_MAPPING = {
        common_binding.StringObjectPropertyType: String,
        common_binding.IntegerObjectPropertyType: Integer,
        common_binding.PositiveIntegerObjectPropertyType: PositiveInteger,
        common_binding.UnsignedIntegerObjectPropertyType: UnsignedInteger,
        common_binding.UnsignedLongObjectPropertyType: UnsignedLong,
        common_binding.AnyURIObjectPropertyType: AnyURI,
        common_binding.HexBinaryObjectPropertyType: HexBinary,
        common_binding.DateTimeObjectPropertyType: DateTime,
        common_binding.DateObjectPropertyType: Date,
        common_binding.TimeObjectPropertyType: Time,
        common_binding.DurationObjectPropertyType: Duration,
        common_binding.NonNegativeIntegerObjectPropertyType: NonNegativeInteger,
        common_binding.FloatObjectPropertyType: Float,
        common_binding.DoubleObjectPropertyType: Double,
        common_binding.LongObjectPropertyType: Long,
        # This shouldn't be needed anymore, but we'll leave it here to be safe.
        common_binding.SimpleHashValueType: HexBinary,
#        common_binding.HashNameType: HashName,
    }



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import datetime

from mixbox import entities
from mixbox import fields

import cybox
import cybox.bindings.cybox_core as core_binding
from cybox.common import StructuredText, MeasureSource
from cybox.common.vocabs import VocabField
from cybox.core import ActionReference, AssociatedObject, Frequency

from cybox.common.vocabs import ActionName, ActionType
from cybox.common.vocabs import ActionArgumentName as ArgumentName


class ActionAliases(entities.EntityList):
    _binding = core_binding
    _binding_class = core_binding.ActionAliasesType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'
    action_alias = fields.TypedField("Action_Alias", cybox.Unicode, multiple=True)


class ActionArgument(entities.Entity):
    _binding = core_binding
    _binding_class = core_binding.ActionArgumentType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    argument_name = VocabField("Argument_Name", ArgumentName)
    argument_value = fields.TypedField("Argument_Value")


class ActionArguments(entities.EntityList):
    _binding_class = core_binding.ActionArgumentsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'
    action_argument = fields.TypedField("Action_Argument", ActionArgument, multiple=True)


class AssociatedObjects(entities.EntityList):
    _binding_class = core_binding.AssociatedObjectsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'
    associated_object = fields.TypedField("Associated_Object", AssociatedObject, multiple=True)

class ActionRelationship(entities.Entity):
    _binding = core_binding
    _binding_class = _binding.ActionRelationshipType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    type = VocabField("Type", ActionType)
    action_references = fields.TypedField("Action_Reference", ActionReference, multiple=True)


class ActionRelationships(entities.EntityList):
    _binding_class = core_binding.ActionRelationshipsType
    _binding_var = "Relationship"
    _contained_type = ActionRelationship
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'
    relationship = fields.TypedField("Relationship", ActionRelationship, multiple=True)

class Action(entities.Entity):
    _binding = core_binding
    _binding_class = core_binding.ActionType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    id_ = fields.TypedField("id")
    idref = fields.TypedField("idref")
    ordinal_position = fields.TypedField("ordinal_position")
    action_status = fields.TypedField("action_status")
    context = fields.TypedField("context")
    timestamp = fields.DateTimeField("timestamp")
    type_ = VocabField("Type", ActionType)
    name = VocabField("Name", ActionName)
    description = fields.TypedField("Description", StructuredText)
    action_aliases = fields.TypedField("Action_Aliases", ActionAliases)
    action_arguments = fields.TypedField("Action_Arguments", ActionArguments)
    discovery_method = fields.TypedField("Discovery_Method", MeasureSource)
    associated_objects = fields.TypedField("Associated_Objects", AssociatedObjects)
    relationships = fields.TypedField("Relationships", ActionRelationships)
    frequency = fields.TypedField("Frequency", Frequency)


class Actions(entities.EntityList):
    _binding_class = core_binding.ActionsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'
    action = fields.TypedField("Action", Action, multiple=True)


# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.dns_cache_object import DNSCache, DNSCacheEntry
from cybox.test.objects import ObjectTestCase


class TestAccount(ObjectTestCase, unittest.TestCase):
    object_type = "DNSCacheObjectType"
    klass = DNSCache

    _full_dict = {
        'xsi:type': object_type,
        'dns_cache_entry': [
            {
                'ttl': 60,
                'dns_entry': {
                    'description': "First test",
                    'data_length': 1024,
                    'xsi:type': 'DNSRecordObjectType'
                }
            },
            {
                'ttl': 16,
                'dns_entry': {
                    'description': "Second test",
                    'data_length': 1565,
                    'record_name': "A record name",
                    'xsi:type': 'DNSRecordObjectType'
                }
            },
        ]
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_mailslot_object as win_mailslot_binding
from cybox.common import NonNegativeInteger, ObjectProperties, String
from cybox.objects.win_handle_object import WinHandle


class WinMailslot(ObjectProperties):
    _binding = win_mailslot_binding
    _binding_class = win_mailslot_binding.WindowsMailslotObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-mailslot-2"
    _XSI_NS = "WinMailslotObj"
    _XSI_TYPE = "WindowsMailslotObjectType"

    handle = fields.TypedField("Handle", WinHandle)
    max_message_size = fields.TypedField("Max_Message_Size", NonNegativeInteger)
    name = fields.TypedField("Name", String)
    read_timeout = fields.TypedField("Read_Timeout", NonNegativeInteger)
    security_attributes = fields.TypedField("Security_Attributes", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import logging
import unittest

from mixbox.vendor.six import u

from cybox.common import Hash, HashList, HashName, HexBinary
import cybox.test

logger = logging.getLogger(__name__)

EMPTY_MD5 = u("d41d8cd98f00b204e9800998ecf8427e")
EMPTY_SHA1 = u("da39a3ee5e6b4b0d3255bfef95601890afd80709")
EMPTY_SHA224 = u("d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f")
EMPTY_SHA256 = \
        u("e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855")
EMPTY_SHA384 = u(
        "38b060a751ac96384cd9327eb1b1e36a21fdb71114be0743"
        "4c0cc7bf63f6e1da274edebfe76f65fbd51ad2f14898b95b")
EMPTY_SHA512 = u(
        "cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce"
        "47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e")

TEST_HASH_LIST = [
    {'simple_hash_value': EMPTY_MD5,
        'type': Hash.TYPE_MD5},
    {'simple_hash_value': EMPTY_SHA1,
        'type': Hash.TYPE_SHA1},
    {'simple_hash_value': EMPTY_SHA224,
        'type': Hash.TYPE_SHA224},
    {'simple_hash_value': EMPTY_SHA256,
        'type': Hash.TYPE_SHA256},
    {'simple_hash_value': EMPTY_SHA384,
        'type': Hash.TYPE_SHA384},
    {'simple_hash_value': EMPTY_SHA512,
        'type': Hash.TYPE_SHA512},
]


class TestHash(unittest.TestCase):

    def setUp(self):
        self.md5 = HexBinary(EMPTY_MD5)
        self.sha1 = HexBinary(EMPTY_SHA1)
        self.sha224 = HexBinary(EMPTY_SHA224)
        self.sha256 = HexBinary(EMPTY_SHA256)
        self.sha384 = HexBinary(EMPTY_SHA384)
        self.sha512 = HexBinary(EMPTY_SHA512)

    def test_autotype(self):
        h = Hash()
        h.simple_hash_value = "0123456789abcdef0123456789abcdef"
        self.assertEqual(h.type_, Hash.TYPE_MD5)
        h.type_ = Hash.TYPE_OTHER
        self.assertEqual(h.type_, Hash.TYPE_OTHER)
        h.simple_hash_value = "0123456789abcdef0123456789abcdef"
        self.assertEqual(h.type_, Hash.TYPE_OTHER)

        h2 = Hash()
        h2.type_ = Hash.TYPE_OTHER
        h2.simple_hash_value = "0123456789abcdef0123456789abcdef"
        self.assertEqual(h2.type_, Hash.TYPE_OTHER)

    def test_autotype_md5(self):
        """32-character hash is assumed to be MD5"""
        h = Hash(EMPTY_MD5)
        self.assertEqual(h.type_, Hash.TYPE_MD5)

        h2 = Hash(self.md5)
        self.assertEqual(h2.type_, Hash.TYPE_MD5)

    def test_autotype_sha1(self):
        """40-character hash is assumed to be SHA-1"""
        h = Hash(EMPTY_SHA1)
        self.assertEqual(h.type_, Hash.TYPE_SHA1)

        h2 = Hash(self.sha1)
        self.assertEqual(h2.type_, Hash.TYPE_SHA1)

    def test_autotype_sha224(self):
        """56-character hash is assumed to be SHA-224"""
        h = Hash(EMPTY_SHA224)
        self.assertEqual(h.type_, Hash.TYPE_SHA224)

        h2 = Hash(self.sha224)
        self.assertEqual(h2.type_, Hash.TYPE_SHA224)

    def test_autotype_sha256(self):
        """64-character hash is assumed to be SHA-256"""
        h = Hash(EMPTY_SHA256)
        self.assertEqual(h.type_, Hash.TYPE_SHA256)

        h2 = Hash(self.sha256)
        self.assertEqual(h2.type_, Hash.TYPE_SHA256)

    def test_autotype_sha384(self):
        """96-character hash is assumed to be SHA-384"""
        h = Hash(EMPTY_SHA384)
        self.assertEqual(h.type_, Hash.TYPE_SHA384)

        h2 = Hash(self.sha384)
        self.assertEqual(h2.type_, Hash.TYPE_SHA384)

    def test_autotype_sha512(self):
        """128-character hash is assumed to be SHA-512"""
        h = Hash(EMPTY_SHA512)
        self.assertEqual(h.type_, Hash.TYPE_SHA512)

        h2 = Hash(self.sha512)
        self.assertEqual(h2.type_, Hash.TYPE_SHA512)

    def test_autotype_other(self):
        """Hash of unknown length is assigned TYPE_OTHER"""
        h = Hash("0123456789abcdef")
        self.assertEqual(h.type_, Hash.TYPE_OTHER)

    def test_round_trip(self):
        t = HashName(Hash.TYPE_MD5)

        h = Hash(self.md5, t)

        hash2 = cybox.test.round_trip(h)

        self.assertEqual(hash2.simple_hash_value, self.md5)
        #TODO: make this really pass
        self.assertEqual(hash2.type_.value, t.value)

    def test_round_trip2(self):
        hash_dict = {'simple_hash_value': EMPTY_MD5,
                     'type': Hash.TYPE_MD5}
        hash_obj = Hash.object_from_dict(hash_dict)
        hash_dict2 = Hash.dict_from_object(hash_obj)
        self.assertEqual(hash_dict, hash_dict2)

    def test_xml_output(self):
        h = Hash(self.md5)
        h2 = cybox.test.round_trip(h)
        self.assertEqual(str(h2), EMPTY_MD5)

        s = h2.to_xml()
        self.assertTrue(EMPTY_MD5.encode("utf-8") in s)

    def test_constructor(self):
        s = HexBinary(EMPTY_MD5)
        h = Hash(s)

    def test_exact_hash(self):
        h = Hash(EMPTY_MD5, exact=True)
        self.assertEqual("Equals", h.simple_hash_value.condition)
        self.assertEqual("Equals", h.type_.condition)


class TestHashList(unittest.TestCase):

    def test_round_trip(self):
        hashlist_list = TEST_HASH_LIST
        hashlist_obj = HashList.object_from_list(hashlist_list)
        hashlist_list2 = HashList.list_from_object(hashlist_obj)
        self.assertEqual(hashlist_list, hashlist_list2)

    def test_hash_lookup(self):
        h = HashList()
        self.assertEqual(None, h.md5)
        self.assertEqual(None, h.sha1)
        self.assertEqual(None, h.sha224)
        self.assertEqual(None, h.sha256)
        self.assertEqual(None, h.sha384)
        self.assertEqual(None, h.sha512)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.code_object as code_binding
from cybox.common import (ObjectProperties, String, HexBinary, StructuredText,
        MeasureSource, PlatformSpecification, ExtractedFeatures, DigitalSignatureList)


class CodeSegmentXOR(String):
    _binding = code_binding
    _binding_class = code_binding.CodeSegmentXORType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/code-2"

    xor_pattern = fields.TypedField('xor_pattern')

    def is_plain(self):
        return super(CodeSegmentXOR, self).is_plain() and not self.xor_pattern


class TargetedPlatforms(entities.EntityList):
    _binding = code_binding
    _binding_class = code_binding.TargetedPlatformsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/code-2"
    targeted_platform = fields.TypedField("Targeted_Platform", PlatformSpecification, multiple=True)


class Code(ObjectProperties):
    _binding = code_binding
    _binding_class = code_binding.CodeObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/code-2"
    _XSI_NS = "CodeObj"
    _XSI_TYPE = "CodeObjectType"

    description = fields.TypedField('Description', StructuredText)
    type_ = fields.TypedField('Type', String)
    purpose = fields.TypedField('Purpose', String)
    code_language = fields.TypedField('Code_Language', String)
    targeted_platforms = fields.TypedField('Targeted_Platforms', TargetedPlatforms)
    processor_family = fields.TypedField('Processor_Family', String, multiple=True)
    discovery_method = fields.TypedField('Discovery_Method', MeasureSource)
    start_address = fields.TypedField('Start_Address', HexBinary)
    code_segment = fields.TypedField('Code_Segment', String)
    code_segment_xor = fields.TypedField('Code_Segment_XOR', CodeSegmentXOR)
    digital_signatures = fields.TypedField('Digital_Signatures', DigitalSignatureList)
    extracted_features = fields.TypedField('Extracted_Features', ExtractedFeatures)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_waitable_timer_object as win_waitable_timer_binding
from cybox.common import String, ObjectProperties


class WinWaitableTimer(ObjectProperties):
    _binding = win_waitable_timer_binding
    _binding_class = win_waitable_timer_binding.WindowsWaitableTimerObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-waitable-timer-2"
    _XSI_NS = "WinWaitableTimerObj"
    _XSI_TYPE = "WindowsWaitableTimerObjectType"

    security_attributes = fields.TypedField("Security_Attributes", String)
    name = fields.TypedField("Name", String)
    type_ = fields.TypedField("Type", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.file_object import File, FilePath, Packer, SymLinksList

from cybox.common import Hash, String
from cybox.compat import long
import cybox.test
from cybox.test import EntityTestCase
from cybox.test.common.hash_test import (EMPTY_MD5, EMPTY_SHA1, EMPTY_SHA224,
        EMPTY_SHA256, EMPTY_SHA384, EMPTY_SHA512)
from cybox.test.objects import ObjectTestCase


class TestFilePath(unittest.TestCase):

    def setUp(self):
        self.path = "C:\\WINDOWS\\system32\\"
        self.path_bytes = b"C:\\WINDOWS\\system32\\"

    def test_round_trip(self):
        fp = FilePath(self.path)
        fp.fully_qualified = True

        fp2 = cybox.test.round_trip(fp, FilePath)
        self.assertEqual(fp.to_dict(), fp2.to_dict())

    def test_xml_output(self):
        fp = FilePath(self.path)

        self.assertTrue(self.path_bytes in fp.to_xml())


class TestFile(ObjectTestCase, unittest.TestCase):
    object_type = "FileObjectType"
    klass = File

    _full_dict = {
        'is_packed': False,
        'is_masqueraded': True,
        'file_name': u("example.txt"),
        'file_path': {'value': u("C:\\Temp"),
                      'fully_qualified': True},
        'device_path': u("\\Device\\CdRom0"),
        'full_path': u("C:\\Temp\\example.txt"),
        'file_extension': u("txt"),
        'size_in_bytes': { 'apply_condition': 'ANY', 'condition':'InclusiveBetween', 'value': [long(1023), long(1024)] },
        'magic_number': u("D0CF11E0"),
        'file_format': u("ASCII Text"),
        'hashes': [
            {
                'type': Hash.TYPE_MD5,
                'simple_hash_value': u("0123456789abcdef0123456789abcdef")
            }
        ],
        'digital_signatures': [
            {
                'certificate_issuer': u("Microsoft"),
                'certificate_subject': u("Notepad"),
            }
        ],
        'modified_time': "2010-11-06T02:02:02+08:00",
        'accessed_time': "2010-11-07T02:03:02+09:00",
        'created_time': "2010-11-08T02:04:02+10:00",
        'user_owner': u("sballmer"),
        'packer_list': [
            {
                'name': u("UPX"),
                'version': u("3.91"),
            }
        ],
        'peak_entropy': 7.454352453,
        'sym_links': [u("../link_destination")],
        'byte_runs': [{'offset': 16, 'byte_run_data': u("1A2B3C4D")}],
        'extracted_features': {
            'strings': [{'string_value': u("string from the file")}],
        },
        'encryption_algorithm': u("RC4"),
        'compression_method': u("deflate"),
        'compression_version': u("1.0"),
        'compression_comment': u("This has been compressed"),
        'xsi:type': object_type,
    }

    def test_filepath_is_none(self):
        # This would throw an exception at one point. Should be fixed now.
        a = File.from_dict({'file_name': 'abcd.dll'})

    def test_get_hashes(self):
        f = File()
        f.add_hash(Hash(EMPTY_MD5))
        f.add_hash(Hash(EMPTY_SHA1))
        f.add_hash(Hash(EMPTY_SHA224))
        f.add_hash(Hash(EMPTY_SHA256))
        f.add_hash(Hash(EMPTY_SHA384))
        f.add_hash(Hash(EMPTY_SHA512))

        self.assertEqual(EMPTY_MD5, f.md5)
        self.assertEqual(EMPTY_SHA1, f.sha1)
        self.assertEqual(EMPTY_SHA224, f.sha224)
        self.assertEqual(EMPTY_SHA256, f.sha256)
        self.assertEqual(EMPTY_SHA384, f.sha384)
        self.assertEqual(EMPTY_SHA512, f.sha512)

    def test_set_hashes(self):
        f = File()
        f.md5 = EMPTY_MD5
        f.sha1 = EMPTY_SHA1
        f.sha224 = EMPTY_SHA224
        f.sha256 = EMPTY_SHA256
        f.sha384 = EMPTY_SHA384
        f.sha512 = EMPTY_SHA512

        self.assertEqual(EMPTY_MD5, f.md5)
        self.assertEqual(EMPTY_SHA1, f.sha1)
        self.assertEqual(EMPTY_SHA224, f.sha224)
        self.assertEqual(EMPTY_SHA256, f.sha256)
        self.assertEqual(EMPTY_SHA384, f.sha384)
        self.assertEqual(EMPTY_SHA512, f.sha512)

    def test_add_hash_string(self):
        s = "ffffffffffffffffffff"
        f = File()
        f.add_hash(s)

        h = f.hashes[0]
        self.assertEqual(s, str(h.simple_hash_value))
        self.assertEqual(Hash.TYPE_OTHER, h.type_)

    def test_fields(self):
        f = File()
        f.file_name = "blah.exe"
        self.assertEqual(String, type(f.file_name))

        f.file_path = "C:\\Temp"
        self.assertEqual(FilePath, type(f.file_path))

    def test_fields_not_shared(self):
        # In a previous version of TypedFields, all objects of the same type
        # shared a single value of each field. Obviously this was a mistake.
        f = File()
        f.file_name = "README.txt"
        self.assertEqual("README.txt", f.file_name)

        f2 = File()
        self.assertEqual(None, f2.file_name)

    def test_file_name_delimiter(self):
        f = File()
        f.file_name = ["foo", "bar"]
        f.file_name.delimiter = "^^"
        self.assertTrue(b"foo^^bar" in f.to_xml())


class TestPacker(EntityTestCase, unittest.TestCase):
    klass = Packer

    _full_dict = {
        'name': u("CrazyPack"),
        'version': u("2.0.1"),
        'entry_point': u("EB0FA192"),
        'signature': u("xxCrAzYpAcKxx"),
        'type': u("Protector"),
        'ep_jump_codes': {
            'depth': 2,
            'opcodes': u("A B C")
        },
        'detected_entrypoint_signatures': [
              {
                  'name': u("test 1"),
                  'type' : u('type 1')
              },
              {
                  'name': u("test 2"),
                  'type' : u('type 2')
              }
        ],
    }


class TestSymLinksList(EntityTestCase, unittest.TestCase):
    klass = SymLinksList

    _full_dict = [
            "C:\\Temp\\Recent\\link_destination",
            "C:\\Temp\\Recent\\link_destination2",
    ]

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Common utility methods"""

#importlib is imported below
import os

from mixbox.vendor import six

from .caches import *
from .nsparser import *


def denormalize_from_xml(value, delimiter):
    if not delimiter:
        raise ValueError("delimiter must not be None")

    if value is None:
        return None

    # This is probably not necessary since the parser will have removed
    # the CDATA already.
    denormalized = unwrap_cdata(value)

    if delimiter in denormalized:
        return denormalized.split(delimiter)
    else:
        return denormalized


def normalize_to_xml(value, delimiter):
    if value is None:
        return None

    if not delimiter:
        raise ValueError("delimiter must not be None")

    if isinstance(value, list):
        normalized_list = [six.text_type(x) for x in value]
        if any(delimiter in x for x in normalized_list):
            raise ValueError("list items cannot contain delimiter")
        normalized = delimiter.join(normalized_list)
    else:
        normalized = six.text_type(value)
        if delimiter in normalized:
            raise ValueError("value cannot contain delimiter")

    return normalized


def wrap_cdata(value):
    return "<![CDATA[" + value + "]]>"


def unwrap_cdata(value):
    """Remove CDATA wrapping from `value` if present"""
    if value.startswith("<![CDATA[") and value.endswith("]]>"):
        return value[9:-3]
    else:
        return value


def _import_submodules(pkg):
    import importlib
    filename = pkg.__file__
    if "__init__.py" not in filename:
        return
    for module in os.listdir(os.path.dirname(filename)):
        if "__init__.py" in module or not module.endswith(".py"):
            continue
        mod_name = "%s.%s" % (pkg.__name__, module[:-3])
        importlib.import_module(mod_name)


def _import_all():
    """Import all modules in the core, common and objects packages.

    This is useful when we want to check all classes for some property.
    """
    # Everything in common should be imported by cybox.common.__init__
    import cybox.common
    # Everything in core should be imported by cybox.core.__init__
    import cybox.core
    import cybox.objects
    _import_submodules(cybox.objects)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.cybox_common as common_binding
import cybox.objects

from .properties import String


class Property(String):
    _binding = common_binding
    _binding_class = _binding.PropertyType

    name = fields.TypedField("name")
    description = fields.TypedField("description")

    def is_plain(self):
        """Whether the Property can be represented as a single value."""
        return (
            self.name is None and
            self.description is None and
            super(Property, self).is_plain()
        )


class CustomProperties(entities.EntityList):
    _binding = common_binding
    _binding_class = common_binding.CustomPropertiesType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'
    property_ = fields.TypedField("Property", Property, multiple=True)


class ObjectPropertiesFactory(entities.EntityFactory):
    @classmethod
    def objkey(cls, obj):
        xsi_type = obj.xsi_type

        if not xsi_type:
            raise ValueError("No xsi:type found on ObjectProperties instance.")

        return xsi_type.split(":")[1]

    @classmethod
    def entity_class(cls, key):
        if not key:
            raise ValueError("Must provide an xsi:type key for ObjectProperties.")
        return cybox.objects.get_class_for_object_type(key)


class ObjectProperties(entities.Entity):
    """The Cybox ObjectProperties base class."""
    _XSI_TYPE = None
    _XSI_NS   = None
    _binding = common_binding
    _binding_class = _binding.ObjectPropertiesType

    object_reference = fields.TypedField("object_reference")
    custom_properties = fields.TypedField("Custom_Properties", CustomProperties)

    def __init__(self):
        super(ObjectProperties, self).__init__()
        self.parent = None

    @property
    def parent(self):
        import cybox.core

        if not self._parent:
            self._parent = cybox.core.Object(self)
        return self._parent

    @parent.setter
    def parent(self, value):
        import cybox.core

        if value and not isinstance(value, cybox.core.Object):
            raise ValueError("Must be an Object")
        self._parent = value

    def add_related(self, related, relationship, inline=True):
        self.parent.add_related(related, relationship, inline)

    def to_obj(self, ns_info=None):
        obj = super(ObjectProperties, self).to_obj(ns_info=ns_info)

        if self._XSI_TYPE and self._XSI_NS:
            obj.xsi_type = "%s:%s" % (self._XSI_NS, self._XSI_TYPE)

        return obj

    def to_dict(self):
        d = super(ObjectProperties, self).to_dict()

        if self._XSI_TYPE:
            d['xsi:type'] = self._XSI_TYPE

        return d



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_filemapping_object as win_filemapping_binding
from cybox.objects.win_handle_object import WinHandle
from cybox.common import ObjectProperties, String, UnsignedLong


class WinFilemapping(ObjectProperties):
    _binding = win_filemapping_binding
    _binding_class = win_filemapping_binding.WindowsFilemappingObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-filemapping-1"
    _XSI_NS = "WinFilemappingObj"
    _XSI_TYPE = "WindowsFilemappingObjectType"

    name = fields.TypedField("Name", String)
    file_handle = fields.TypedField("File_Handle", WinHandle)
    handle = fields.TypedField("Handle", WinHandle)
    page_protection_value = fields.TypedField("Page_Protection_Value", String)
    page_protection_attribute = fields.TypedField("Page_Protection_Attribute", String, multiple=True)
    maximum_size = fields.TypedField("Maximum_Size", UnsignedLong)
    actual_size = fields.TypedField("Actual_Size", UnsignedLong)
    security_attributes = fields.TypedField("Security_Attributes", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.port_object import Port
from cybox.test.objects import ObjectTestCase


class TestPort(ObjectTestCase, unittest.TestCase):
    object_type = "PortObjectType"
    klass = Port

    _full_dict = {
        'port_value': 80,
        'layer4_protocol': 'TCP',
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
from mixbox import entities
from mixbox import fields

import cybox.bindings.cybox_common as common_binding
from cybox.common import Integer, String


class DataSize(String):
    _binding = common_binding
    _binding_class = common_binding.DataSizeType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    units = fields.TypedField("units")

    def is_plain(self):
        return (super(DataSize, self).is_plain() and self.units is None)


class DataSegment(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.DataSegmentType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    id_ = fields.TypedField("id")
    data_format = fields.TypedField("Data_Format")
    data_size = fields.TypedField("Data_Size", DataSize)
    byte_order = fields.TypedField("Byte_Order", String)
    data_segment = fields.TypedField("Data_Segment", String)
    offset = fields.TypedField("Offset", Integer)
    search_distance = fields.TypedField("Search_Distance", Integer)
    search_within = fields.TypedField("Search_Within", Integer)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.common import ByteRun, ByteRuns, Hash
import cybox.test
from cybox.test import EntityTestCase


class TestByteRun(EntityTestCase, unittest.TestCase):
    klass = ByteRun

    _full_dict = {
        'offset': 1000,
        'file_system_offset': 1024,
        'image_offset': 512,
        'length': 10,
        'hashes': [{'type': Hash.TYPE_MD5,
                    'simple_hash_value':
                        '0123456789abcdef0123456789abcdef'}],
        'byte_run_data': "helloworld",
    }


class TestByteRuns(unittest.TestCase):

    def test_round_trip(self):
        byteruns_list = [
                        {'byte_run_data': "a",
                         'length': 1},
                        {'byte_run_data': "blahblah",
                         'length': 8},
                        {'byte_run_data': "aeiou",
                         'length': 5},
                      ]
        byteruns_list2 = cybox.test.round_trip_list(ByteRuns, byteruns_list)
        self.assertEqual(byteruns_list, byteruns_list2)

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.network_connection_object as network_connection_binding
from cybox.common import ObjectProperties, String, DateTime
from cybox.objects.socket_address_object import SocketAddress
from cybox.objects.http_session_object import HTTPSession
from cybox.objects.dns_query_object import DNSQuery


class Layer7Connections(entities.Entity):
    _binding = network_connection_binding
    _binding_class = network_connection_binding.Layer7ConnectionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-connection-2"

    http_session = fields.TypedField("HTTP_Session", HTTPSession)
    dns_query = fields.TypedField("DNS_Query", DNSQuery, multiple=True)


class NetworkConnection(ObjectProperties):
    _binding = network_connection_binding
    _binding_class = network_connection_binding.NetworkConnectionObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-connection-2"
    _XSI_NS = "NetworkConnectionObj"
    _XSI_TYPE = "NetworkConnectionObjectType"

    tls_used = fields.TypedField('tls_used')
    creation_time = fields.TypedField('Creation_Time', DateTime)
    layer3_protocol = fields.TypedField('Layer3_Protocol', String)
    layer4_protocol = fields.TypedField('Layer4_Protocol', String)
    layer7_protocol = fields.TypedField('Layer7_Protocol', String)
    source_socket_address = fields.TypedField('Source_Socket_Address',
                                             SocketAddress)
    source_tcp_state = fields.TypedField('Source_TCP_State')
    destination_socket_address = fields.TypedField('Destination_Socket_Address',
                                                  SocketAddress)
    destination_tcp_state = fields.TypedField('Destination_TCP_State')
    layer7_connections = fields.TypedField('Layer7_Connections',
                                          Layer7Connections)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.disk_object as disk_binding
from cybox.objects.disk_partition_object import DiskPartition
from cybox.common import ObjectProperties, String, UnsignedLong


class PartitionList(entities.EntityList):
    _binding = disk_binding
    _binding_class = disk_binding.PartitionListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/disk-2"
    partition = fields.TypedField("Partition", DiskPartition, multiple=True)


class Disk(ObjectProperties):
    _binding = disk_binding
    _binding_class = disk_binding.DiskObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/disk-2"
    _XSI_NS = "DiskObj"
    _XSI_TYPE = "DiskObjectType"

    disk_name = fields.TypedField('Disk_Name', String)
    disk_size = fields.TypedField('Disk_Size', UnsignedLong)
    free_space = fields.TypedField('Free_Space', UnsignedLong)
    partition_list = fields.TypedField('Partition_List', PartitionList)
    type_ = fields.TypedField('Type', String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest
import cybox.bindings.cybox_common as common_binding
import cybox.bindings.mutex_object as mutex_binding
from cybox.objects.mutex_object import Mutex


class MutexTest(unittest.TestCase):
    def setUp(self):
        self.test_dict = {'named': True, 'name': 'test_name'}
        self.mutex_obj = Mutex.object_from_dict(self.test_dict)
        self.mutex_dict = Mutex.dict_from_object(self.mutex_obj)

    def test_obj_from_dict(self):
        #Make sure it's an instance of the right class
        self.assertTrue(isinstance(self.mutex_obj,
                                   mutex_binding.MutexObjectType))
        #Test the named attribute
        self.assertEqual(self.mutex_obj.named, True)
        #Test the name element
        self.assertTrue(isinstance(self.mutex_obj.Name,
                                   common_binding.StringObjectPropertyType))
        self.assertEqual(self.mutex_obj.Name.valueOf_, 'test_name')

    def test_dict_from_obj(self):
        #Make sure it's an instance of the right class
        self.assertTrue(isinstance(self.mutex_dict, dict))
        #Test the dictionary values
        self.assertEqual(self.mutex_dict['named'], True)
        self.assertEqual(self.mutex_dict['name'], 'test_name')

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.compat import long
from cybox.objects.win_filemapping_object import WinFilemapping
from cybox.test.objects import ObjectTestCase


class TestWinFilemapping(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsFilemappingObjectType"
    klass = WinFilemapping

    _full_dict = {
        'handle': {
            'id': 1234,
            'name': u("MyHandle"),
            'type': u("Window"),
            'object_address': long(0xdeadbeef),
            'access_mask': long(0x70000000),
            'pointer_count': long(3),
            'xsi:type': "WindowsHandleObjectType",
        },
        'file_handle': {
            'id': 5678,
            'name': u("MyHandle2"),
            'type': u("Window"),
            'object_address': long(0xbeadbeef),
            'access_mask': long(0x90009000),
            'pointer_count': long(9),
            'xsi:type': "WindowsHandleObjectType",
        },
        'security_attributes': u("Attributes go here"),
        'name': "A mapping name",
        'maximum_size': 1000,
        'actual_size': 250,
        'page_protection_value': "a protection value",
        'page_protection_attribute': [
            "a protection attribute",
            "another attribute"
        ],
        'xsi:type': object_type
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.x509_certificate_object as x509_certificate_binding
from cybox.common import ObjectProperties, String, Integer, DateTime, NonNegativeInteger


class Validity(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.ValidityType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    not_before = fields.TypedField("Not_Before", DateTime)
    not_after = fields.TypedField("Not_After", DateTime)

class RSAPublicKey(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.RSAPublicKeyType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    modulus = fields.TypedField("Modulus", String)
    exponent = fields.TypedField("Exponent", Integer)


class SubjectPublicKey(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.SubjectPublicKeyType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    public_key_algorithm = fields.TypedField("Public_Key_Algorithm", String)
    rsa_public_key = fields.TypedField("RSA_Public_Key", RSAPublicKey)


class X509V3Extensions(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.X509V3ExtensionsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    basic_constraints = fields.TypedField("Basic_Constraints", String)
    name_constraints = fields.TypedField("Name_Constraints", String)
    policy_constraints = fields.TypedField("Policy_Constraints", String)
    key_usage = fields.TypedField("Key_Usage", String)
    extended_key_usage = fields.TypedField("Extended_Key_Usage", String)
    subject_key_identifier = fields.TypedField("Subject_Key_Identifier", String)
    authority_key_identifier = fields.TypedField("Authority_Key_Identifier", String)
    subject_alternative_name = fields.TypedField("Subject_Alternative_Name", String)
    issuer_alternative_name = fields.TypedField("Issuer_Alternative_Name", String)
    subject_directory_attributes = fields.TypedField("Subject_Directory_Attributes", String)
    crl_distribution_points = fields.TypedField("CRL_Distribution_Points", String)
    inhibit_any_policy = fields.TypedField("Inhibit_Any_Policy", NonNegativeInteger)
    private_key_usage_period = fields.TypedField("Private_Key_Usage_Period", Validity)
    certificate_policies = fields.TypedField("Certificate_Policies", String)
    policy_mappings = fields.TypedField("Policy_Mappings", String)


class X509NonStandardExtensions(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.X509NonStandardExtensionsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    netscape_comment = fields.TypedField("Netscape_Comment", String)
    netscape_certificate_type = fields.TypedField("Netscape_Certificate_Type", String)
    old_authority_key_identifier = fields.TypedField("Old_Authority_Key_Identifier", String)
    old_primary_key_attributes = fields.TypedField("Old_Primary_Key_Attributes", String)


class X509Cert(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.X509CertificateContentsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    version = fields.TypedField("Version", Integer)
    serial_number = fields.TypedField("Serial_Number", String)
    signature_algorithm = fields.TypedField("Signature_Algorithm", String)
    issuer = fields.TypedField("Issuer", String)
    validity = fields.TypedField("Validity", Validity)
    subject = fields.TypedField("Subject", String)
    subject_public_key = fields.TypedField("Subject_Public_Key", SubjectPublicKey)
    standard_extensions = fields.TypedField("Standard_Extensions", X509V3Extensions)
    non_standard_extensions = fields.TypedField("Non_Standard_Extensions", X509NonStandardExtensions)


class X509CertificateSignature(entities.Entity):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.X509CertificateSignatureType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'

    signature_algorithm = fields.TypedField("Signature_Algorithm", String)
    signature = fields.TypedField("Signature", String)


class X509Certificate(ObjectProperties):
    _binding = x509_certificate_binding
    _binding_class = x509_certificate_binding.X509CertificateObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/x509-certificate-2'
    _XSI_NS = "X509CertificateObj"
    _XSI_TYPE = "X509CertificateObjectType"

    certificate = fields.TypedField("Certificate", X509Cert)
    raw_certificate = fields.TypedField("Raw_Certificate", String)
    certificate_signature = fields.TypedField("Certificate_Signature", X509CertificateSignature)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.memory_object import Memory

from cybox.compat import long
from cybox.test.common.hash_test import EMPTY_MD5
from cybox.test.objects import ObjectTestCase


class TestMemory(ObjectTestCase, unittest.TestCase):
    object_type = "MemoryObjectType"
    klass = Memory

    _full_dict = {
        'custom_properties': [
            {'name': "Prop1", 'description': "Property1", 'value': "Value1"},
            {'name': "Prop2", 'description': "Property2", 'value': "Value2"},
        ],
        'is_injected': True,
        'is_mapped': False,
        'is_protected': True,
        'is_volatile': False,
        'hashes': [{'type': u("MD5"), 'simple_hash_value': EMPTY_MD5}],
        'name': u("A memory region"),
        'region_size': long(65536),
        'memory_source': u(".data"),
        'block_type': u("Free"),
        'region_start_address': u("00040000"),
        'region_end_address': u("00048000"),
        'extracted_features': {'functions': [u("StringA"), u("StringB")]},
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import file_object


class ImageFileFormatType(cybox_common.BaseObjectPropertyType):
    """The ImageFileFormatType specifies image file formats via a union of
    the ImageFileFormatEnum type and the atomic xs:string type. Its
    base type is the CybOX Core cybox_common.BaseObjectPropertyType, for
    permitting complex (i.e. regular-expression based)
    specifications."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(ImageFileFormatType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_, )
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if ImageFileFormatType.subclass:
            return ImageFileFormatType.subclass(*args_, **kwargs_)
        else:
            return ImageFileFormatType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(ImageFileFormatType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='ImageFileObj:', name_='ImageFileFormatType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='ImageFileFormatType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='ImageFileObj:', name_='ImageFileFormatType'):
        super(ImageFileFormatType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='ImageFileFormatType')
    def exportChildren(self, lwrite, level, namespace_='ImageFileObj:', name_='ImageFileFormatType', fromsubclass_=False, pretty_print=True):
        super(ImageFileFormatType, self).exportChildren(lwrite, level, 'ImageFileObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(ImageFileFormatType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class ImageFileFormatType

class ImageFileObjectType(file_object.FileObjectType):
    """The ImageFileObjectType type is intended to characterize image
    files.The image_is_compressed field specifies whether the image
    in the image file is compressed."""

    subclass = None
    superclass = file_object.FileObjectType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, is_packed=None, is_masqueraded=None, File_Name=None, File_Path=None, Device_Path=None, Full_Path=None, File_Extension=None, Size_In_Bytes=None, Magic_Number=None, File_Format=None, Hashes=None, Digital_Signatures=None, Modified_Time=None, Accessed_Time=None, Created_Time=None, File_Attributes_List=None, Permissions=None, User_Owner=None, Packer_List=None, Peak_Entropy=None, Sym_Links=None, Byte_Runs=None, Extracted_Features=None, Encryption_Algorithm=None, Decryption_Key=None, Compression_Method=None, Compression_Version=None, Compression_Comment=None, image_is_compressed=None, Image_File_Format=None, Image_Height=None, Image_Width=None, Bits_Per_Pixel=None, Compression_Algorithm=None):
        super(ImageFileObjectType, self).__init__(object_reference, Custom_Properties, xsi_type, is_packed, is_masqueraded, File_Name, File_Path, Device_Path, Full_Path, File_Extension, Size_In_Bytes, Magic_Number, File_Format, Hashes, Digital_Signatures, Modified_Time, Accessed_Time, Created_Time, File_Attributes_List, Permissions, User_Owner, Packer_List, Peak_Entropy, Sym_Links, Byte_Runs, Extracted_Features, Encryption_Algorithm, Decryption_Key, Compression_Method, Compression_Version, Compression_Comment, )
        self.image_is_compressed = _cast(bool, image_is_compressed)
        self.Image_File_Format = Image_File_Format
        self.Image_Height = Image_Height
        self.Image_Width = Image_Width
        self.Bits_Per_Pixel = Bits_Per_Pixel
        self.Compression_Algorithm = Compression_Algorithm
    def factory(*args_, **kwargs_):
        if ImageFileObjectType.subclass:
            return ImageFileObjectType.subclass(*args_, **kwargs_)
        else:
            return ImageFileObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Image_File_Format(self): return self.Image_File_Format
    def set_Image_File_Format(self, Image_File_Format): self.Image_File_Format = Image_File_Format
    def validate_ImageFileFormatType(self, value):
        # Validate type ImageFileFormatType, a restriction on None.
        pass
    def get_Image_Height(self): return self.Image_Height
    def set_Image_Height(self, Image_Height): self.Image_Height = Image_Height
    def validate_IntegerObjectPropertyType(self, value):
        # Validate type cybox_common.IntegerObjectPropertyType, a restriction on None.
        pass
    def get_Image_Width(self): return self.Image_Width
    def set_Image_Width(self, Image_Width): self.Image_Width = Image_Width
    def get_Bits_Per_Pixel(self): return self.Bits_Per_Pixel
    def set_Bits_Per_Pixel(self, Bits_Per_Pixel): self.Bits_Per_Pixel = Bits_Per_Pixel
    def validate_PositiveIntegerObjectPropertyType(self, value):
        # Validate type cybox_common.PositiveIntegerObjectPropertyType, a restriction on None.
        pass
    def get_Compression_Algorithm(self): return self.Compression_Algorithm
    def set_Compression_Algorithm(self, Compression_Algorithm): self.Compression_Algorithm = Compression_Algorithm
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_image_is_compressed(self): return self.image_is_compressed
    def set_image_is_compressed(self, image_is_compressed): self.image_is_compressed = image_is_compressed
    def hasContent_(self):
        if (
            self.Image_File_Format is not None or
            self.Image_Height is not None or
            self.Image_Width is not None or
            self.Bits_Per_Pixel is not None or
            self.Compression_Algorithm is not None or
            super(ImageFileObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='ImageFileObj:', name_='ImageFileObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='ImageFileObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='ImageFileObj:', name_='ImageFileObjectType'):
        super(ImageFileObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='ImageFileObjectType')
        if self.image_is_compressed is not None:

            lwrite(' image_is_compressed="%s"' % self.gds_format_boolean(self.image_is_compressed, input_name='image_is_compressed'))
    def exportChildren(self, lwrite, level, namespace_='ImageFileObj:', name_='ImageFileObjectType', fromsubclass_=False, pretty_print=True):
        super(ImageFileObjectType, self).exportChildren(lwrite, level, 'ImageFileObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Image_File_Format is not None:
            self.Image_File_Format.export(lwrite, level, 'ImageFileObj:', name_='Image_File_Format', pretty_print=pretty_print)
        if self.Image_Height is not None:
            self.Image_Height.export(lwrite, level, 'ImageFileObj:', name_='Image_Height', pretty_print=pretty_print)
        if self.Image_Width is not None:
            self.Image_Width.export(lwrite, level, 'ImageFileObj:', name_='Image_Width', pretty_print=pretty_print)
        if self.Bits_Per_Pixel is not None:
            self.Bits_Per_Pixel.export(lwrite, level, 'ImageFileObj:', name_='Bits_Per_Pixel', pretty_print=pretty_print)
        if self.Compression_Algorithm is not None:
            self.Compression_Algorithm.export(lwrite, level, 'ImageFileObj:', name_='Compression_Algorithm', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('image_is_compressed', node)
        if value is not None:

            if value in ('true', '1'):
                self.image_is_compressed = True
            elif value in ('false', '0'):
                self.image_is_compressed = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        super(ImageFileObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Image_File_Format':
            obj_ = ImageFileFormatType.factory()
            obj_.build(child_)
            self.set_Image_File_Format(obj_)
        elif nodeName_ == 'Image_Height':
            obj_ = cybox_common.IntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Image_Height(obj_)
        elif nodeName_ == 'Image_Width':
            obj_ = cybox_common.IntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Image_Width(obj_)
        elif nodeName_ == 'Bits_Per_Pixel':
            obj_ = cybox_common.PositiveIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Bits_Per_Pixel(obj_)
        elif nodeName_ == 'Compression_Algorithm':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Compression_Algorithm(obj_)
        super(ImageFileObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class ImageFileObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Byte_Order': cybox_common.EndiannessType,
    'Errors': cybox_common.ErrorsType,
    'File_Extension': cybox_common.StringObjectPropertyType,
    'Time': cybox_common.TimeType,
    'Sym_Links': file_object.SymLinksListType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Size_In_Bytes': cybox_common.UnsignedLongObjectPropertyType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Compression_Version': cybox_common.StringObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Byte_Runs': cybox_common.ByteRunsType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Device_Path': cybox_common.StringObjectPropertyType,
    'Produced_Time': cybox_common.DateTimeWithPrecisionType,
    'Reference': cybox_common.ToolReferenceType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Bits_Per_Pixel': cybox_common.PositiveIntegerObjectPropertyType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Start_Date': cybox_common.DateWithPrecisionType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'Start_Time': cybox_common.DateTimeWithPrecisionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Version': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Created_Time': cybox_common.DateTimeObjectPropertyType,
    'Type': file_object.PackerClassType,
    'Compilers': cybox_common.CompilersType,
    'Digital_Signatures': cybox_common.DigitalSignaturesType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'File_Format': cybox_common.StringObjectPropertyType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Detected_Entrypoint_Signatures': file_object.EntryPointSignatureListType,
    'Tool_Hashes': cybox_common.HashListType,
    'File_Path': file_object.FilePathType,
    'Observable_Location': cybox_common.LocationType,
    'Entry_Point_Signature': file_object.EntryPointSignatureType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'End_Date': cybox_common.DateWithPrecisionType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Compensation_Model': cybox_common.CompensationModelType,
    'Signature': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'User_Owner': cybox_common.StringObjectPropertyType,
    'Contributors': cybox_common.PersonnelType,
    'Packer': file_object.PackerType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'File_Attributes_List': file_object.FileAttributeType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Received_Time': cybox_common.DateTimeWithPrecisionType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Packer_List': file_object.PackerListType,
    'Import': cybox_common.StringObjectPropertyType,
    'Accessed_Time': cybox_common.DateTimeObjectPropertyType,
    'Sym_Link': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Permissions': file_object.FilePermissionsType,
    'Compression_Algorithm': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Opcodes': cybox_common.StringObjectPropertyType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Encryption_Algorithm': cybox_common.CipherType,
    'Compilation_Date': cybox_common.DateTimeWithPrecisionType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'EP_Jump_Codes': file_object.EPJumpCodeType,
    'File_Name': cybox_common.StringObjectPropertyType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Data_Size': cybox_common.DataSizeType,
    'Observation_Location': cybox_common.LocationType,
    'Modified_Time': cybox_common.DateTimeObjectPropertyType,
    'Image_Width': cybox_common.IntegerObjectPropertyType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Extracted_Features': cybox_common.ExtractedFeaturesType,
    'Magic_Number': cybox_common.HexBinaryObjectPropertyType,
    'Image_Height': cybox_common.IntegerObjectPropertyType,
    'Decryption_Key': cybox_common.StringObjectPropertyType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Full_Path': cybox_common.StringObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Depth': cybox_common.IntegerObjectPropertyType,
    'Entry_Point': cybox_common.HexBinaryObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compression_Comment': cybox_common.StringObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Compression_Method': cybox_common.StringObjectPropertyType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'End_Time': cybox_common.DateTimeWithPrecisionType,
    'File': file_object.FileObjectType,
    'Contributor': cybox_common.ContributorType,
    'Peak_Entropy': cybox_common.DoubleObjectPropertyType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Image_File'
        rootClass = ImageFileObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout.write, 0, name_=rootTag,
        namespacedef_='',
        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Image_File'
        rootClass = ImageFileObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Image_File'
        rootClass = ImageFileObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout.write, 0, name_="Image_File",
        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "ImageFileObjectType",
    "ImageFileFormatType"
    ]
